<?php

namespace Drupal\persian_fields\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'melli_code' field type.
 *
 * @FieldType(
 *   category="Persian Fields",
 *   id = "iran_postal_code",
 *   label = @Translation("Iran postal code"),
 *   description = @Translation("Postal code for Iranians"),
 *   default_widget = "iran_postal_code_widget",
 *   default_formatter = "iran_postal_code_formatter"
 * )
 */
class IranPostalCode extends FieldItemBase
{

    /**
     * {@inheritdoc}
     */
    public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition)
    {
        // Prevent early t() calls by using the TranslatableMarkup.
        $properties['value'] = DataDefinition::create('string')
            ->setLabel(new TranslatableMarkup('Iran postal code'))
            ->setSetting('case_sensitive', $field_definition->getSetting('case_sensitive'))
            ->setRequired(TRUE);

        return $properties;
    }

    /**
     * {@inheritdoc}
     */
    public static function schema(FieldStorageDefinitionInterface $field_definition)
    {
        $schema = [
            'columns' => [
                'value' => [
                    'type' => 'varchar',
                    'length' => 10,
                ],
            ],
        ];

        return $schema;
    }

    /**
     * {@inheritdoc}
     */
    public function getConstraints()
    {
        $constraints = parent::getConstraints();

        $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();

        $constraints[] = $constraint_manager->create('ComplexData', [
            'value' => [
                'iran_postal_code' => [],
            ],
        ]);

        return $constraints;
    }

    /**
     * {@inheritdoc}
     */
    public static function generateSampleValue(FieldDefinitionInterface $field_definition)
    {
        $values['value'] = '9583255858';
        return $values;
    }

    /**
     * {@inheritdoc}
     */
    public function isEmpty()
    {
        $value = $this->get('value')->getValue();
        return $value === NULL || $value === '';
    }

}
